@@ -15,7 +15,25 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 """PyTorch dataset loader repository"""
 from .create_dataloader import create_dataloader
+from .create_dataset import create_dataset
 from .encode_features import encode_features
 from .load_dataset import load_dataset
 
 __author__ = "Abien Fred Agarap"
+
+
+def list_datasets() -> str:
+    datasets = [
+        "MNIST",
+        "Fashion-MNIST",
+        "EMNIST",
+        "KMNIST",
+        "SVHN",
+        "CIFAR10",
+        "AG News" "20 Newsgroups",
+        "Malware Image",
+        "Wisconsin Diagnostic Breast Cancer",
+        "Binary COVID19",
+        "Multi COVID19",
+    ]
+    return datasets
